@@ -66,27 +66,31 @@ char *nr2signal(int sig) {
                  break;
         case 16: signame = "SIGURG";
                  break;
-        default: signame = gm_malloc(20);
+        default: signame = malloc(20);
                  snprintf(signame, 20, "signal %d", sig);
                  return signame;
                  break;
     }
-    return gm_strdup(signame);
+    return strdup(signame);
 }
 
 
 /* extract check result */
 char *extract_check_result(FILE *fp, int trimmed) {
-    char *output;
-    char *escaped;
+    int size;
+    char buffer[GM_BUFFERSIZE] = "";
+    char output[GM_BUFFERSIZE] = "";
 
-    output   = gm_malloc(sizeof(char*)*GM_BUFFERSIZE);
-    output[0]='\x0';
-    read_filepointer(&output, fp);
+    /* get all lines of plugin output - escape newlines */
+    strcpy(buffer,"");
+    strcpy(output,"");
+    size = GM_MAX_OUTPUT;
+    while(size > 0 && fgets(buffer,sizeof(buffer)-1,fp)){
+        strncat(output, buffer, size);
+        size -= strlen(buffer);
+    }
 
-    escaped  = gm_escape_newlines(output, trimmed);
-    free(output);
-    return(escaped);
+    return(gm_escape_newlines(output, trimmed));
 }
 
 
@@ -97,37 +101,8 @@ int run_check(char *processed_command, char **ret, char **err) {
     pid_t pid;
     int pipe_stdout[2], pipe_stderr[2], pipe_rwe[3];
     int retval;
-    int i;
-    int restricted_ok = FALSE;
     sigset_t mask;
 
-    /* verify restricted paths
-     * make sure our command does not contain any bash special characters
-     * and starts with one of the allowed paths
-     */
-    if(mod_gm_opt->restrict_path_num) {
-        if(*processed_command != '/') {
-            *err = gm_strdup("");
-            gm_asprintf(ret, "ERROR: restricted paths in affect, but command does not start with an absolute path: %.*s...\n", 8, processed_command);
-            return(GM_EXIT_UNKNOWN);
-        }
-        if(strpbrk(processed_command,"$&();<>`\"'|") != NULL) {
-            *err = gm_strdup("");
-            gm_asprintf(ret, "ERROR: restricted paths in affect, but command contains forbidden character(s): %.*s...\n", 8, processed_command);
-            return(GM_EXIT_UNKNOWN);
-        }
-        for(i=0;i<mod_gm_opt->restrict_path_num;i++) {
-            if(starts_with(mod_gm_opt->restrict_path[i], processed_command)) {
-                restricted_ok = TRUE;
-            }
-        }
-        if(!restricted_ok) {
-            *err = gm_strdup("");
-            gm_asprintf(ret, "ERROR: command does not start with any of the restricted paths: %.*s...\n", 8, processed_command);
-            return(GM_EXIT_UNKNOWN);
-        }
-    }
-
 #ifdef EMBEDDEDPERL
     retval = run_epn_check(processed_command, ret, err);
     if(retval != GM_NO_EPN) {
@@ -140,7 +115,7 @@ int run_check(char *processed_command, char **ret, char **err) {
      * and cmd must begin with a /. Otherwise "BLAH=BLUB cmd" would lead
      * to file not found errors
      */
-    if((*processed_command == '/' || *processed_command == '.') && strpbrk(processed_command,"!$^&*()~[]\\|{};<>?`\"'") == NULL) {
+    if((*processed_command == '/' || *processed_command == '.') && !strpbrk(processed_command,"!$^&*()~[]\\|{};<>?`\"'")) {
         /* use the fast execvp when there are no shell characters */
         gm_log( GM_LOG_TRACE, "using execvp, no shell characters found\n" );
 
@@ -248,14 +223,20 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
     int pipe_stdout[2] , pipe_stderr[2];
     int return_code;
     int pclose_result;
-    int x;
-    char *plugin_output, *plugin_error, *bufdup;
+    int size;
+    char *plugin_output, *plugin_error;
+    char *bufdup;
+    char buffer[GM_BUFFERSIZE], buf_error[GM_BUFFERSIZE], source[GM_BUFFERSIZE];
     struct timeval start_time,end_time;
     pid_t pid    = 0;
+    buffer[0]    = '\x0';
+    buf_error[0] = '\x0';
+    source[0]    = '\x0';
 
-    gm_log( GM_LOG_TRACE, "execute_safe_command(%d, %s)\n", exec_job->timeout, exec_job->command_line );
+    gm_log( GM_LOG_TRACE, "execute_safe_command()\n" );
 
-    /* mark all filehandles to close on exec */
+    // mark all filehandles to close on exec
+    int x;
     for(x = 0; x<=64; x++)
         fcntl(x, F_SETFD, FD_CLOEXEC);
 
@@ -280,7 +261,7 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
         if( pid == -1 ) {
             if(exec_job->output != NULL)
                 free(exec_job->output);
-            exec_job->output      = gm_strdup("(Error On Fork)");
+            exec_job->output      = strdup("(Error On Fork)");
             exec_job->return_code = 3;
             return(GM_ERROR);
         }
@@ -322,6 +303,12 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
             free(plugin_error);
             _exit(return_code);
         }
+        else {
+            snprintf( buffer,    sizeof( buffer )-1,    "%s", plugin_output );
+            snprintf( buf_error, sizeof( buf_error )-1, "%s", plugin_error  );
+            free(plugin_output);
+            free(plugin_error);
+        }
     }
 
     /* we are the parent */
@@ -336,33 +323,30 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
             waitpid(pid, &return_code, 0);
             gm_log( GM_LOG_TRACE, "finished check from pid: %d with status: %d\n", pid, return_code);
             /* get all lines of plugin output */
-            plugin_output = gm_malloc(GM_BUFFERSIZE);
-            plugin_output[0]='\x0';
-            plugin_error  = gm_malloc(GM_BUFFERSIZE);
-            plugin_error[0]='\x0';
-            read_pipe(&plugin_output, pipe_stdout[0]);
-            read_pipe(&plugin_error, pipe_stderr[0]);
+            if((size = read(pipe_stdout[0], buffer, sizeof(buffer)-1)) < 0)
+                perror("read");
+            buffer[size] = '\0';
+            if((size = read(pipe_stderr[0], buf_error, sizeof(buf_error)-1)) < 0)
+                perror("read");
+            buf_error[size] = '\0';
         }
         return_code = real_exit_code(return_code);
 
         /* file not executable? */
         if(return_code == 126) {
             return_code = STATE_CRITICAL;
-            free(plugin_output);
-            gm_asprintf(&plugin_output, "CRITICAL: Return code of 126 is out of bounds. Make sure the plugin you're trying to run is executable. (worker: %s)", identifier);
+            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of 126 is out of bounds. Make sure the plugin you're trying to run is executable. (worker: %s)", identifier);
         }
         /* file not found errors? */
         else if(return_code == 127) {
             return_code = STATE_CRITICAL;
-            free(plugin_output);
-            gm_asprintf(&plugin_output, "CRITICAL: Return code of 127 is out of bounds. Make sure the plugin you're trying to run actually exists. (worker: %s)", identifier);
+            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of 127 is out of bounds. Make sure the plugin you're trying to run actually exists. (worker: %s)", identifier);
         }
         /* signaled */
         else if(return_code >= 128 && return_code < 144) {
             char * signame = nr2signal((int)(return_code-128));
-            bufdup = gm_strdup(plugin_output);
-            free(plugin_output);
-            gm_asprintf(&plugin_output, "CRITICAL: Return code of %d is out of bounds. Plugin exited by signal %s. (worker: %s)\\n%s", (int)(return_code), signame, identifier, bufdup);
+            bufdup = strdup(buffer);
+            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. Plugin exited by signal %s. (worker: %s)\\n%s", (int)(return_code), signame, identifier, bufdup);
             return_code = STATE_CRITICAL;
             free(bufdup);
             free(signame);
@@ -370,18 +354,21 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
         /* other error codes > 3 */
         else if(return_code > 3) {
             gm_log( GM_LOG_DEBUG, "check exited with exit code > 3. Exit: %d\n", (int)(return_code));
-            gm_log( GM_LOG_DEBUG, "stdout: %s\n", plugin_output);
-            bufdup = gm_strdup(plugin_output);
-            free(plugin_output);
-            gm_asprintf(&plugin_output, "CRITICAL: Return code of %d is out of bounds. (worker: %s)\\n%s", (int)(return_code), identifier, bufdup);
+            gm_log( GM_LOG_DEBUG, "stdout: %s\n", buffer);
+            bufdup = strdup(buffer);
+            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. (worker: %s)\\n%s", (int)(return_code), identifier, bufdup);
             free(bufdup);
             if(return_code != 25 && mod_gm_opt->workaround_rc_25 == GM_DISABLED) {
                 return_code = STATE_CRITICAL;
             }
         }
 
-        exec_job->output      = plugin_output;
-        exec_job->error       = plugin_error;
+        if(exec_job->output != NULL)
+            free(exec_job->output);
+        if(exec_job->error != NULL)
+            free(exec_job->error);
+        exec_job->output      = strdup(buffer);
+        exec_job->error       = strdup(buf_error);
         exec_job->return_code = return_code;
         if( fork_exec == GM_ENABLED) {
             close(pipe_stdout[0]);
@@ -400,15 +387,19 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
     if(exec_job->timeout < ((int)end_time.tv_sec - (int)exec_job->start_time.tv_sec)) {
         exec_job->return_code   = mod_gm_opt->timeout_return;
         exec_job->early_timeout = 1;
+        if ( !strcmp( exec_job->type, "service" ) )
+            snprintf( buffer, sizeof( buffer ) -1, "(Service Check Timed Out On Worker: %s)", identifier);
+        if ( !strcmp( exec_job->type, "host" ) )
+            snprintf( buffer, sizeof( buffer ) -1, "(Host Check Timed Out On Worker: %s)", identifier);
         free(exec_job->output);
-        if ( !strcmp( exec_job->type, "service" ) ) {
-            gm_asprintf(&exec_job->output, "(Service Check Timed Out On Worker: %s)", identifier);
-        }
-        else {
-            gm_asprintf(&exec_job->output, "(Host Check Timed Out On Worker: %s)", identifier);
-        }
+        exec_job->output = strdup( buffer );
     }
 
+    snprintf( source, sizeof( source )-1, "Mod-Gearman Worker @ %s", identifier);
+    if(exec_job->source != NULL)
+        free(exec_job->source);
+    exec_job->source = strdup(source);
+
     return(GM_OK);
 }
 
@@ -448,7 +439,7 @@ void check_alarm_handler(int sig) {
     signal(SIGINT, SIG_DFL);
     sleep(1);
 
-    /* skip sigkill in test mode */
+    // skip sigkill in test mode
     if(getenv("MODGEARMANTEST") == NULL) {
         gm_log( GM_LOG_TRACE, "send SIGKILL to %d\n", pid);
         kill(-pid, SIGKILL);
@@ -503,7 +494,7 @@ void send_timeout_result(gm_job_t * exec_job) {
     if ( !strcmp( exec_job->type, "host" ) )
         snprintf( buffer, sizeof( buffer ) -1, "(Host Check Timed Out On Worker: %s)\n", mod_gm_opt->identifier);
     free(exec_job->output);
-    exec_job->output = gm_strdup( buffer );
+    exec_job->output = strdup( buffer );
 
     send_result_back(exec_job);
 
@@ -527,7 +518,7 @@ void send_failed_result(gm_job_t * exec_job, int sig) {
     signame = nr2signal(sig);
     snprintf( buffer, sizeof( buffer )-1, "(Return code of %d is out of bounds. Worker exited by signal %s on worker: %s)", sig, signame, mod_gm_opt->identifier);
     free(exec_job->output);
-    exec_job->output = gm_strdup( buffer );
+    exec_job->output = strdup( buffer );
     free(signame);
 
     send_result_back(exec_job);
