@@ -26,9 +26,9 @@
  * \section intro_sec Introduction
  *
  * Mod-Gearman (http://labs.consol.de/nagios/mod-gearman) is a new
- * way of distributing active Nagios checks (http://www.nagios.org) across your network. It
+ * way of distributing active Naemon checks (http://www.naemon.org) across your network. It
  * consists of two parts: There is a NEB module which resides in the
- * Nagios core and adds servicechecks, hostchecks and eventhandler to a
+ * Naemon core and adds servicechecks, hostchecks and eventhandler to a
  * Gearman (http://gearman.org) queue. There can be multiple equal
  * gearman servers.  The counterpart is one or more worker clients for
  * the checks itself. They can be bound to host and servicegroups.
@@ -46,7 +46,6 @@
  */
 
 #include <config.h>
-#include <gm_alloc.h>
 #include <stdio.h>
 #include <sys/time.h>
 #include <arpa/inet.h>
@@ -55,11 +54,11 @@
 #define MOD_GM_COMMON_H
 
 /* constants */
-#define GM_VERSION                 "1.5.4"
+#define GM_VERSION                 "2.0.0"
 #define GM_ENABLED                      1
 #define GM_DISABLED                     0
-#define GM_BUFFERSIZE               65536
-#define GM_MAX_OUTPUT             1048576   /* limit plugin output size to 1mb */
+#define GM_BUFFERSIZE               98304
+#define GM_MAX_OUTPUT               65536   /* must be ~30% below GM_BUFFERSIZE for base64/encryption */
 #define GM_LISTSIZE                   512
 #define GM_NEBTYPESSIZE                33   /* maximum number of neb types */
 
@@ -70,10 +69,6 @@
 #define GM_ERROR                        1
 #define GM_NO_EPN                      -1
 
-#define GM_PERFDATA_ALL                 2
-
-#define GM_EXIT_UNKNOWN               768   /* results in exit code 3 after processed by WEXITSTATUS() */
-
 /* log modes */
 #define GM_LOG_ERROR                   -1
 #define GM_LOG_INFO                     0
@@ -155,7 +150,7 @@
  */
 typedef struct mod_gm_export {
     char   * name[GM_LISTSIZE];             /**< list of queue names to export into */
-    int      return_code[GM_LISTSIZE];      /**< list of return codes which should be returned to nagios */
+    int      return_code[GM_LISTSIZE];      /**< list of return codes which should be returned to naemon */
     int      elem_number;                   /**< number of elements */
 } mod_gm_exp_t;
 
@@ -195,7 +190,7 @@ typedef struct mod_gm_opt_struct {
     int            job_timeout;                             /**< override job timeout */
     int            encryption;                              /**< flag wheter messages are encrypted */
     int            transportmode;                           /**< flag for the transportmode, base64 only or base64 and encrypted  */
-    int            logmode;                                 /**< logmode: auto, syslog, file or nagios */
+    int            logmode;                                 /**< logmode: auto, syslog, file or core */
     char         * logfile;                                 /**< path for the logfile */
     FILE         * logfile_fp;                              /**< filedescriptor for the logfile */
     int            use_uniq_jobs;                           /**< flag whether normal jobs will be sent with/without uniq set */
@@ -241,9 +236,6 @@ typedef struct mod_gm_opt_struct {
     int            use_perl_cache;                          /**< cache embedded perl scripts */
     char         * p1_file;                                 /**< path to p1 file, needed for embedded perl */
 #endif
-    char         * restrict_path[GM_LISTSIZE];              /**< list of path restrictions */
-    int            restrict_path_num;                       /**< number of path restrictions */
-    char         * restrict_command_characters;             /**< forbidden characters in command lines */
     int            workaround_rc_25;                        /**< optional workaround for plugins returning exit code 25 */
 /* send_gearman */
     int            timeout;                                 /**< timeout for waiting reading on stdin */
@@ -271,6 +263,7 @@ typedef struct gm_job_struct {
     char         * result_queue;        /**< name of the result queue */
     char         * output;              /**< output from the executed command line (stdout) */
     char         * error;               /**< errors from the executed command line (stderr) */
+    char         * source;              /**< source of this check */
     int            return_code;         /**< return code for this job */
     int            early_timeout;       /**< did the check run into a timeout */
     int            check_options;       /**< check_options given from the core */
