@@ -30,19 +30,15 @@
 NEB_API_VERSION( CURRENT_NEB_API_VERSION )
 
 /* import some global variables */
-extern int            event_broker_options;
+extern unsigned long  event_broker_options;
 extern int            currently_running_host_checks;
 extern int            currently_running_service_checks;
 extern int            service_check_timeout;
 extern int            host_check_timeout;
-extern timed_event  * event_list_low;
-extern timed_event  * event_list_low_tail;
 extern int            process_performance_data;
-extern check_result   check_result_info;
-extern check_result * check_result_list;
 
 /* global variables */
-static check_result * mod_gm_result_list = 0;
+static objectlist * mod_gm_result_list = 0;
 static pthread_mutex_t mod_gm_result_list_mutex = PTHREAD_MUTEX_INITIALIZER;
 void *gearman_module_handle=NULL;
 gearman_client_st client;
@@ -65,7 +61,6 @@ static void  set_target_queue( host *, service * );
 static int   handle_process_events( int, void * );
 static int   handle_timed_events( int, void * );
 static void  start_threads(void);
-static check_result * merge_result_lists(check_result * lista, check_result * listb);
 static void move_results_to_core(void);
 
 int nebmodule_init( int flags, char *args, nebmodule *handle ) {
@@ -85,7 +80,7 @@ int nebmodule_init( int flags, char *args, nebmodule *handle ) {
     neb_set_module_info( gearman_module_handle, NEBMODULE_MODINFO_LICENSE, "GPL v3" );
     neb_set_module_info( gearman_module_handle, NEBMODULE_MODINFO_DESC,    "distribute host/service checks and eventhandler via gearman" );
 
-    mod_gm_opt = gm_malloc(sizeof(mod_gm_opt_t));
+    mod_gm_opt = malloc(sizeof(mod_gm_opt_t));
     set_default_options(mod_gm_opt);
 
     /* parse arguments */
@@ -277,56 +272,31 @@ static int handle_timed_events( int event_type, void *data ) {
 }
 
 
-/* merge results with core */
-static check_result * merge_result_lists(check_result * lista, check_result * listb) {
-    check_result * result = 0;
-
-    check_result ** iter;
-    for (iter = &result; lista && listb; iter = &(*iter)->next) {
-        if (mod_gm_time_compare(&lista->finish_time, &listb->finish_time) <= 0) {
-            *iter = lista; lista = lista->next;
-        } else {
-            *iter = listb; listb = listb->next;
-        }
-    }
-
-    *iter = lista? lista: listb;
-
-    return result;
-}
-
-
 /* insert results list into core */
 static void move_results_to_core() {
-   check_result * local;
+    /* safely save off currently local list */
+    pthread_mutex_lock(&mod_gm_result_list_mutex);
 
-   /* safely save off currently local list */
-   pthread_mutex_lock(&mod_gm_result_list_mutex);
-   local = mod_gm_result_list;
-   mod_gm_result_list = 0;
-   pthread_mutex_unlock(&mod_gm_result_list_mutex);
+    objectlist *tmp_list = NULL;
+    for(mod_gm_result_list; mod_gm_result_list; mod_gm_result_list = mod_gm_result_list->next) {
+        free(tmp_list);
+        process_check_result(mod_gm_result_list->object_ptr);
+        free_check_result(mod_gm_result_list->object_ptr);
+        free(mod_gm_result_list->object_ptr);
+        tmp_list = mod_gm_result_list;
+    }
+    mod_gm_result_list = 0;
+    free(tmp_list);
 
-   /* merge local into check_result_list, store in check_result_list */
-   check_result_list = merge_result_lists(local, check_result_list);
+    pthread_mutex_unlock(&mod_gm_result_list_mutex);
 }
 
 
 /* add list to gearman result list */
 void mod_gm_add_result_to_list(check_result * newcr) {
-   check_result ** curp;
-
-   assert(newcr);
-
-   pthread_mutex_lock(&mod_gm_result_list_mutex);
-
-   for (curp = &mod_gm_result_list; *curp; curp = &(*curp)->next)
-      if (mod_gm_time_compare(&(*curp)->finish_time, &newcr->finish_time) >= 0)
-         break;
-
-   newcr->next = *curp;
-   *curp = newcr;
-
-   pthread_mutex_unlock(&mod_gm_result_list_mutex);
+    pthread_mutex_lock(&mod_gm_result_list_mutex);
+    add_object_to_objectlist(&mod_gm_result_list, newcr);
+    pthread_mutex_unlock(&mod_gm_result_list_mutex);
 }
 
 
@@ -345,7 +315,7 @@ static int handle_process_events( int event_type, void *data ) {
         send_now = TRUE;
 
         /* verify names of supplied groups
-         * this cannot be done befor nagios has finished reading his config
+         * this cannot be done befor naemon has finished reading his config
          * verify local servicegroups names
          */
         while ( mod_gm_opt->local_servicegroups_list[x] != NULL ) {
@@ -475,7 +445,7 @@ static int handle_eventhandler( int event_type, void *data ) {
         gm_log( GM_LOG_TRACE, "handle_eventhandler() finished unsuccessfully\n" );
     }
 
-    /* tell nagios to not execute */
+    /* tell naemon to not execute */
     return NEBERROR_CALLBACKOVERRIDE;
 }
 
@@ -539,7 +509,7 @@ static int handle_host_check( int event_type, void *data ) {
     hst->is_being_freshened=FALSE;
 
     /* adjust host check attempt */
-    adjust_host_check_attempt_3x(hst,TRUE);
+    adjust_host_check_attempt(hst,TRUE);
 
     temp_buffer[0]='\x0';
 
@@ -548,7 +518,7 @@ static int handle_host_check( int event_type, void *data ) {
     grab_host_macros(hst);
 
     /* get the raw command line */
-    get_raw_command_line(hst->check_command_ptr,hst->host_check_command,&raw_command,0);
+    get_raw_command_line(hst->check_command_ptr,hst->check_command,&raw_command,0);
     if(raw_command==NULL){
         gm_log( GM_LOG_ERROR, "Raw check command for host '%s' was NULL - aborting.\n",hst->name );
         return NEBERROR_CALLBACKCANCEL;
@@ -619,16 +589,16 @@ static int handle_host_check( int event_type, void *data ) {
     /* orphaned check - submit fake result to mark host as orphaned */
     if(mod_gm_opt->orphan_host_checks == GM_ENABLED && check_options & CHECK_OPTION_ORPHAN_CHECK) {
         gm_log( GM_LOG_DEBUG, "host check for %s orphaned\n", hst->name );
-        if ( ( chk_result = ( check_result * )gm_malloc( sizeof *chk_result ) ) == 0 )
+        if ( ( chk_result = ( check_result * )malloc( sizeof *chk_result ) ) == 0 )
             return NEBERROR_CALLBACKCANCEL;
         snprintf( temp_buffer,GM_BUFFERSIZE-1,"(host check orphaned, is the mod-gearman worker on queue '%s' running?)\n", target_queue);
         init_check_result(chk_result);
-        chk_result->host_name           = gm_strdup( hst->name );
+        chk_result->host_name           = strdup( hst->name );
         chk_result->scheduled_check     = TRUE;
         chk_result->reschedule_check    = TRUE;
         chk_result->output_file         = 0;
         chk_result->output_file_fp      = NULL;
-        chk_result->output              = gm_strdup(temp_buffer);
+        chk_result->output              = strdup(temp_buffer);
         chk_result->return_code         = mod_gm_opt->orphan_return;
         chk_result->check_options       = CHECK_OPTION_NONE;
         chk_result->object_check_type   = HOST_CHECK;
@@ -640,7 +610,7 @@ static int handle_host_check( int event_type, void *data ) {
         chk_result = NULL;
     }
 
-    /* tell nagios to not execute */
+    /* tell naemon to not execute */
     gm_log( GM_LOG_TRACE, "handle_host_check() finished successfully -> %d\n", NEBERROR_CALLBACKOVERRIDE );
     return NEBERROR_CALLBACKOVERRIDE;
 }
@@ -711,7 +681,7 @@ static int handle_svc_check( int event_type, void *data ) {
     grab_service_macros(svc);
 
     /* get the raw command line */
-    get_raw_command_line(svc->check_command_ptr,svc->service_check_command,&raw_command,0);
+    get_raw_command_line(svc->check_command_ptr,svc->check_command,&raw_command,0);
     if(raw_command==NULL){
         gm_log( GM_LOG_ERROR, "Raw check command for service '%s' on host '%s' was NULL - aborting.\n", svc->description, svc->host_name );
         return NEBERROR_CALLBACKCANCEL;
@@ -756,7 +726,7 @@ static int handle_svc_check( int event_type, void *data ) {
     snprintf( uniq,GM_BUFFERSIZE-1,"%s-%s", svcdata->host_name, svcdata->service_description);
 
     /* execute forced checks with high prio as they are propably user requested */
-    if(check_result_info.check_options & CHECK_OPTION_FORCE_EXECUTION)
+    if(svc->check_options & CHECK_OPTION_FORCE_EXECUTION)
         prio = GM_JOB_PRIO_HIGH;
 
     if(add_job_to_queue( &client,
@@ -792,17 +762,17 @@ static int handle_svc_check( int event_type, void *data ) {
     /* orphaned check - submit fake result to mark service as orphaned */
     if(mod_gm_opt->orphan_service_checks == GM_ENABLED && svc->check_options & CHECK_OPTION_ORPHAN_CHECK) {
         gm_log( GM_LOG_DEBUG, "service check for %s - %s orphaned\n", svc->host_name, svc->description );
-        if ( ( chk_result = ( check_result * )gm_malloc( sizeof *chk_result ) ) == 0 )
+        if ( ( chk_result = ( check_result * )malloc( sizeof *chk_result ) ) == 0 )
             return NEBERROR_CALLBACKCANCEL;
         snprintf( temp_buffer,GM_BUFFERSIZE-1,"(service check orphaned, is the mod-gearman worker on queue '%s' running?)\n", target_queue);
         init_check_result(chk_result);
-        chk_result->host_name           = gm_strdup( svc->host_name );
-        chk_result->service_description = gm_strdup( svc->description );
+        chk_result->host_name           = strdup( svc->host_name );
+        chk_result->service_description = strdup( svc->description );
         chk_result->scheduled_check     = TRUE;
         chk_result->reschedule_check    = TRUE;
         chk_result->output_file         = 0;
         chk_result->output_file_fp      = NULL;
-        chk_result->output              = gm_strdup(temp_buffer);
+        chk_result->output              = strdup(temp_buffer);
         chk_result->return_code         = mod_gm_opt->orphan_return;
         chk_result->check_options       = CHECK_OPTION_NONE;
         chk_result->object_check_type   = SERVICE_CHECK;
@@ -814,7 +784,7 @@ static int handle_svc_check( int event_type, void *data ) {
         chk_result = NULL;
     }
 
-    /* tell nagios to not execute */
+    /* tell naemon to not execute */
     gm_log( GM_LOG_TRACE, "handle_svc_check() finished successfully -> %d\n", NEBERROR_CALLBACKOVERRIDE );
 
     return NEBERROR_CALLBACKOVERRIDE;
@@ -834,7 +804,7 @@ static int read_arguments( const char *args_orig ) {
         return GM_ERROR;
     }
 
-    args = gm_strdup(args_orig);
+    args = strdup(args_orig);
     args_c = args;
 
     while ( (ptr = strsep( &args, " " )) != NULL ) {
@@ -1031,6 +1001,7 @@ int handle_perfdata(int event_type, void *data) {
     host *hst        = NULL;
     service *svc     = NULL;
     int has_perfdata = FALSE;
+    char *perf_data;
 
     gm_log( GM_LOG_TRACE, "handle_perfdata(%d)\n", event_type );
     if(process_performance_data == 0) {
@@ -1058,6 +1029,10 @@ int handle_perfdata(int event_type, void *data) {
                 uniq[0]='\x0';
                 snprintf( uniq,GM_BUFFERSIZE-1,"%s", hostchkdata->host_name);
 
+                /* replace newlines with actual newlines */
+                perf_data = replace_str(hostchkdata->perf_data, "\\n", "\n");
+
+
                 temp_buffer[0]='\x0';
                 snprintf( temp_buffer,GM_BUFFERSIZE-1,
                             "DATATYPE::HOSTPERFDATA\t"
@@ -1068,7 +1043,7 @@ int handle_perfdata(int event_type, void *data) {
                             "HOSTSTATE::%d\t"
                             "HOSTSTATETYPE::%d\n",
                             (int)hostchkdata->timestamp.tv_sec,
-                            hostchkdata->host_name, hostchkdata->perf_data,
+                            hostchkdata->host_name, perf_data,
                             hostchkdata->command_name, hostchkdata->command_args,
                             hostchkdata->state, hostchkdata->state_type);
                 has_perfdata = TRUE;
@@ -1083,7 +1058,7 @@ int handle_perfdata(int event_type, void *data) {
                     break;
                 }
 
-                /* find the nagios service object for this service */
+                /* find the naemon service object for this service */
                 svc = find_service(srvchkdata->host_name, srvchkdata->service_description);
                 if(svc->process_performance_data == 0) {
                     gm_log( GM_LOG_TRACE, "handle_perfdata() process_performance_data disabled for: %s - %s\n", svc->host_name, svc->description );
@@ -1093,6 +1068,9 @@ int handle_perfdata(int event_type, void *data) {
                 uniq[0]='\x0';
                 snprintf( uniq,GM_BUFFERSIZE-1,"%s-%s", srvchkdata->host_name, srvchkdata->service_description);
 
+                /* replace newlines with actual newlines */
+                perf_data = replace_str(srvchkdata->perf_data, "\\n", "\n");
+
                 temp_buffer[0]='\x0';
                 snprintf( temp_buffer,GM_BUFFERSIZE-1,
                             "DATATYPE::SERVICEPERFDATA\t"
@@ -1105,8 +1083,9 @@ int handle_perfdata(int event_type, void *data) {
                             "SERVICESTATETYPE::%d\n\n\n",
                             (int)srvchkdata->timestamp.tv_sec,
                             srvchkdata->host_name, srvchkdata->service_description,
-                            srvchkdata->perf_data, svc->service_check_command,
+                            perf_data, svc->check_command,
                             srvchkdata->state, srvchkdata->state_type);
+                free(perf_data);
                 temp_buffer[GM_BUFFERSIZE-1]='\x0';
                 has_perfdata = TRUE;
             }
@@ -1156,20 +1135,6 @@ int handle_export(int callback_type, void *data) {
 
     /* what type of event/data do we have? */
     switch (callback_type) {
-        case NEBCALLBACK_RESERVED0:                         /*  0 */
-            break;
-        case NEBCALLBACK_RESERVED1:                         /*  1 */
-            break;
-        case NEBCALLBACK_RESERVED2:                         /*  2 */
-            break;
-        case NEBCALLBACK_RESERVED3:                         /*  3 */
-            break;
-        case NEBCALLBACK_RESERVED4:                         /*  4 */
-            break;
-        case NEBCALLBACK_RAW_DATA:                          /*  5 */
-            break;
-        case NEBCALLBACK_NEB_DATA:                          /*  6 */
-            break;
         case NEBCALLBACK_PROCESS_DATA:                      /*  7 */
             npd    = (nebstruct_process_data *)data;
             type   = nebtype2str(npd->type);
