@@ -28,6 +28,19 @@
 #include "mod_gearman.h"
 #include "gearman_utils.h"
 
+static const char *gearman_worker_source_name(void *source) {
+    if(!source)
+        return "unknown internal source (voodoo, perhaps?)";
+
+    return (char*) source;
+}
+
+struct check_engine mod_gearman_check_engine = {
+    "Mod-Gearman",
+    gearman_worker_source_name,
+    NULL
+};
+
 /* cleanup and exit this thread */
 static void cancel_worker_thread (void * data) {
 
@@ -124,10 +137,6 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
     }
     mod_gm_decrypt(&decrypted_data, workload, transportmode);
 
-    if(decrypted_data == NULL) {
-        *ret_ptr = GEARMAN_WORK_FAIL;
-        return NULL;
-    }
     gm_log( GM_LOG_TRACE, "%d --->\n%s\n<---\n", strlen(decrypted_data), decrypted_data );
 #ifdef GM_DEBUG
     decrypted_orig   = strdup(decrypted_data);
@@ -154,13 +163,16 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
 #ifdef GM_DEBUG
     free(decrypted_orig);
 #endif
+        free(decrypted_data);
         return NULL;
     }
     init_check_result(chk_result);
     chk_result->scheduled_check     = TRUE;
     chk_result->reschedule_check    = TRUE;
     chk_result->output_file         = 0;
-    chk_result->output_file_fd      = -1;
+    chk_result->output_file_fp      = NULL;
+
+    chk_result->engine              = &mod_gearman_check_engine;
 
     core_start_time.tv_sec          = 0;
     core_start_time.tv_usec         = 0;
@@ -177,7 +189,8 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
                 chk_result->output = strdup("(null)");
             }
             else {
-                chk_result->output = strdup( value );
+                /* replace newlines with actual newlines */
+                chk_result->output = replace_str(value, "\\n", "\n");
             }
         }
 
@@ -188,6 +201,8 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
             chk_result->host_name = strdup( value );
         } else if ( !strcmp( key, "service_description" ) ) {
             chk_result->service_description = strdup( value );
+        } else if ( !strcmp( key, "source" ) ) {
+            chk_result->source = strdup( value );
         } else if ( !strcmp( key, "check_options" ) ) {
             chk_result->check_options = atoi( value );
         } else if ( !strcmp( key, "scheduled_check" ) ) {
@@ -282,6 +297,7 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
     } else {
 #ifdef GM_DEBUG
         /* does this host exist */
+/* TODO: this is done by core already! no need to do this twice */
         host * hst = find_host( chk_result->host_name );
         if(hst == NULL) {
             write_debug_file(&decrypted_orig);
