@@ -260,9 +260,9 @@ void check_logfile(char *logfile, int mode) {
 
     /* cleanup logfile */
     if(errors == 0) {
-        ok(TRUE, "not removed temporary logfile due to errors: %s", logfile);
-    } else {
         ok(unlink(logfile) == 0, "removed temporary logfile: %s", logfile);
+    } else {
+        ok(TRUE, "not removed temporary logfile due to errors: %s", logfile);
     }
 
     free(line);
@@ -367,6 +367,8 @@ void check_no_worker_running(char* worker_logfile) {
     if(rrc != 1) {
         check_logfile(worker_logfile, 3);
     }
+    free(result);
+    free(error);
     return;
 }
 
@@ -375,7 +377,7 @@ void check_no_worker_running(char* worker_logfile) {
 int main (int argc, char **argv, char **env) {
     argc = argc; argv = argv; env  = env;
     int status, chld, rc;
-    int tests = 127;
+    int tests = 125;
     int rrc;
     char cmd[150];
     char *result, *error, *message, *output;
@@ -567,7 +569,6 @@ int main (int argc, char **argv, char **env) {
     /* cleanup */
     free(result);
     free(error);
-    mod_gm_free_opt(mod_gm_opt);
     free_client(&client);
     free_worker(&worker);
 
@@ -599,14 +600,14 @@ int main (int argc, char **argv, char **env) {
         ok(false, "gearmand had to be killed!");
     }
     todo();
-    check_logfile("/tmp/gearmand.log", 2);
+    check_logfile("/tmp/gearmand.log", status != 0 ? 2 : 0);
     endtodo;
+    status = 0;
 
     kill(worker_pid, SIGTERM);
     waitpid(worker_pid, &status, 0);
     ok(status == 0, "worker (%d) exited with exit code %d", worker_pid, real_exit_code(status));
     check_no_worker_running(worker_logfile);
-    check_logfile(worker_logfile, 2);
     status = 0;
 
 #ifdef EMBEDDEDPERL
@@ -615,6 +616,7 @@ int main (int argc, char **argv, char **env) {
 
     free(last_result);
     free(worker_logfile);
+    mod_gm_free_opt(mod_gm_opt);
     endskip;
     return exit_status();
 }
