@@ -182,7 +182,7 @@ void print_usage() {
     printf("\n");
     printf("             [ --host=<hostname>        ]\n");
     printf("             [ --result_queue=<queue>   ]\n");
-    printf("                 default:result_queue    \n");
+    printf("                 default:check_results   \n");
     printf("             [ --timeout=<seconds>      ]\n");
     printf("\n");
     printf("For details see\n");
@@ -215,7 +215,7 @@ int send_result() {
 
     gm_log( GM_LOG_TRACE, "queue: %s\n", mod_gm_opt->result_queue );
     temp_buffer1[0]='\x0';
-    snprintf( temp_buffer1, sizeof( temp_buffer1 )-1, "type=%s\nhost_name=%s\nstart_time=%i.%i\nfinish_time=%i.%i\nreturn_code=%i\n",
+    snprintf( temp_buffer1, sizeof( temp_buffer1 )-1, "type=%s\nhost_name=%s\nstart_time=%i.%i\nfinish_time=%i.%i\nreturn_code=%i\nsource=send_multi\n",
               mod_gm_opt->active == GM_ENABLED ? "active" : "passive",
               mod_gm_opt->host,
               (int)mod_gm_opt->starttime.tv_sec,
@@ -383,6 +383,7 @@ int read_multi_stream(FILE *stream) {
         }
         gm_log( GM_LOG_TRACE, "\tread %ld bytes, %ld bytes remaining in buffer\n", bytes_read, buflen);
     } while (buflen > 0);
+    buffer[buflen] = '\0';
     return count;
 }
 
