@@ -83,9 +83,11 @@ void worker_client(int worker_mode, int indx, int shid) {
     }
 
     /* create duplicate client */
-    if ( create_client_dup( mod_gm_opt->dupserver_list, &client_dup ) != GM_OK ) {
-        gm_log( GM_LOG_ERROR, "cannot start client for duplicate server\n" );
-        _exit( EXIT_FAILURE );
+    if( mod_gm_opt->dupserver_num ) {
+        if ( create_client_dup( mod_gm_opt->dupserver_list, &client_dup ) != GM_OK ) {
+            gm_log( GM_LOG_ERROR, "cannot start client for duplicate server\n" );
+            _exit( EXIT_FAILURE );
+        }
     }
 
 #ifdef EMBEDDEDPERL
@@ -114,6 +116,13 @@ void worker_loop() {
 
         signal(SIGPIPE, SIG_IGN);
         ret = gearman_worker_work( &worker );
+
+        if (mod_gm_opt->max_jobs > 0 && jobs_done >= mod_gm_opt->max_jobs) {
+            gm_log( GM_LOG_TRACE, "jobs done: %i -> exiting...\n", jobs_done );
+            clean_worker_exit(0);
+            _exit( EXIT_SUCCESS );
+        }
+
         if ( ret != GEARMAN_SUCCESS ) {
             gm_log( GM_LOG_ERROR, "worker error: %s\n", gearman_worker_error( &worker ) );
             gearman_job_free_all( &worker );
@@ -191,6 +200,7 @@ void *get_job( gearman_job_st *job, void *context, size_t *result_size, gearman_
 
     if(decrypted_data == NULL) {
         *ret_ptr = GEARMAN_WORK_FAIL;
+        free(decrypted_orig);
         return NULL;
     }
     gm_log( GM_LOG_TRACE, "%d --->\n%s\n<---\n", strlen(decrypted_data), decrypted_data );
@@ -279,12 +289,6 @@ void *get_job( gearman_job_st *job, void *context, size_t *result_size, gearman_
     /* send finish signal to parent */
     set_state(GM_JOB_END);
 
-    if(mod_gm_opt->max_jobs > 0 && jobs_done >= mod_gm_opt->max_jobs) {
-        gm_log( GM_LOG_TRACE, "jobs done: %i -> exiting...\n", jobs_done );
-        clean_worker_exit(0);
-        _exit( EXIT_SUCCESS );
-    }
-
     return NULL;
 }
 
@@ -572,6 +576,7 @@ void *return_status( gearman_job_st *job, void *context, size_t *result_size, ge
         perror("shmat");
         *result_size = 0;
         alarm(0);
+        free(result);
         return NULL;
     }
 
